import DragModal from '@/components/DragModal';
import { Add, Join } from '@/components/DragModal/formdoms';
import Header from '@/components/Header';
import { doFetch } from '@/utils/doFetch';
import { SettingFilled } from '@ant-design/icons';
import { history, Outlet, useModel } from '@umijs/max';
import { useRequest } from 'ahooks';
import { Button, ConfigProvider, Dropdown, message, Popconfirm, Typography } from 'antd';
import zhCN from 'antd/locale/zh_CN';
import dayjs from 'dayjs';
import 'dayjs/locale/zh-cn';
import { useMemo, useRef, useState } from 'react';
import { Scrollbars } from 'react-custom-scrollbars';
import styles from './index.less';

const { Paragraph } = Typography;

dayjs.locale('zh-cn');

function Welcome(props) {
  const {
    initialState: { currentUser, fetchUserInfo },
    setInitialState,
  } = useModel('@@initialState');

  let scrollRef = useRef();
  const [modal, setmodal] = useState({
    open: false,
  });

  const items = useMemo(() => {
    return currentUser?.org_id
      ? [
          {
            key: '4',
            danger: true,
            label: '退出登录',
          },
        ]
      : [
          {
            key: '1',
            label: <a>创建组织</a>,
          },
          {
            key: '2',
            label: <a>加入组织</a>,
          },
          {
            key: '4',
            danger: true,
            label: '退出登录',
          },
        ];
  }, [currentUser?.org_id]);

  const { run, loading } = useRequest(
    () => {
      return fetchUserInfo();
    },
    {
      manual: true,
      onSuccess: (res) => {
        setInitialState((s) => ({
          ...s,
          currentUser: res,
        }));
      },
    },
  );

  return (
    <ConfigProvider
      theme={{
        token: {
          colorPrimary: '#1890ff',
          sizeUnit: 3,
        },
      }}
      locale={zhCN}
    >
      <div className={styles.homepage}>
        <DragModal
          {...modal}
          onCancel={() => {
            setmodal((s) => ({
              ...s,
              open: false,
            }));
          }}
        >
          {modal?.title === '创建组织' && (
            <Add
              refresh={() => {
                run();
                setmodal((s) => ({
                  ...s,
                  open: false,
                }));
              }}
            />
          )}
          {modal?.title === '加入组织' && (
            <Join
              refresh={() => {
                run();
                setmodal((s) => ({
                  ...s,
                  open: false,
                }));
              }}
            />
          )}
        </DragModal>
        <div className={styles.header} id="head">
          <div className="center">
            <Header currentUser={currentUser} run={run}></Header>
            <div className="columns" style={{ paddingLeft: 12 }}>
              <b>{currentUser?.user_name}</b>
              {currentUser?.org_id && (
                <div
                  style={{ fontSize: 12, display: 'flex', alignItems: 'baseline', marginRight: 20 }}
                >
                  <div style={{ flex: 1 }} className="oneline">
                    {currentUser?.org_name}
                  </div>
                  <Paragraph
                    copyable={{
                      text: currentUser?.org_join_key,
                      icon: <a>组织代码</a>,
                      tooltips: '复制组织代码',
                    }}
                    style={{ margin: 0, fontSize: 12 }}
                  ></Paragraph>
                  <Popconfirm
                    title={`是否退出?`}
                    placement="bottomRight"
                    description="退出后将无法查看当前组织的小组"
                    onConfirm={() => {
                      doFetch({
                        url: `/webtool/v1/user/${currentUser?.id}`,
                        params: { org_id: null },
                        method: 'PUT',
                      }).then((res) => {
                        if (res?.code === 0) {
                          message.success('退出成功');
                          history.push('/welcome/homepage');
                          run();
                        }
                      });
                    }}
                  >
                    <a style={{ paddingLeft: 6, color: '#ff4800', cursor: 'pointer' }}>退出组织</a>
                  </Popconfirm>
                </div>
              )}
            </div>
          </div>
          <Dropdown
            menu={{
              items,
              onClick: ({ item, key, keyPath, domEvent }) => {
                switch (key) {
                  case '1':
                    setmodal((s) => ({
                      ...s,
                      open: true,
                      title: '创建组织',
                    }));
                    break;
                  case '2':
                    setmodal((s) => ({
                      ...s,
                      open: true,
                      title: '加入组织',
                    }));
                    break;
                  case '4':
                    localStorage.removeItem('TOKENES');
                    history.push('/user/login');
                    break;
                  default:
                    break;
                }
              },
            }}
            trigger="hover"
            getPopupContainer={() => document.getElementById('head')}
          >
            <Button
              size="large"
              style={{ backgroundColor: 'rgba(255,255,255,0.25)', flexShrink: 0 }}
              icon={<SettingFilled />}
            ></Button>
          </Dropdown>
        </div>
        <div style={{ flex: 1, overflow: 'hidden' }}>
          <Scrollbars
            thumbMinSize={10}
            autoHide
            style={{
              width: '100%',
              height: '100%',
            }}
            hideTracksWhenNotNeeded
            ref={scrollRef}
          >
            <div style={{ padding: 12 }}>
              <Outlet context={{ prop: 'a' }} />
            </div>
          </Scrollbars>
        </div>
      </div>
    </ConfigProvider>
  );
}

export default Welcome;
