/* eslint-disable react/jsx-key */
/* eslint-disable eqeqeq */
import DragModal from '@/components/DragModal';
import { AddPro, Join } from '@/components/DragModal/formdoms';
import IconFont from '@/components/IconFont';
import { doFetch, getFetch } from '@/utils/doFetch';
import {
  ArrowUpOutlined,
  ClockCircleFilled,
  DeleteOutlined,
  PlayCircleFilled,
  PlusOutlined,
  SearchOutlined,
} from '@ant-design/icons';
import { history, useModel } from '@umijs/max';
import { useRequest } from 'ahooks';
import {
  Avatar,
  Card,
  Col,
  Input,
  message,
  Popconfirm,
  Progress,
  Row,
  Skeleton,
  Tooltip,
} from 'antd';
import dayjs from 'dayjs';
import { useState } from 'react';
import './index.less';

let col = { xs: 24, sm: 24, md: 12, lg: 8, xl: 6, xxl: 4 };

const items = {
  0: {
    value: '0',
    statusName: '未开始',
    icon: (
      <Tooltip title="点击开始">
        <PlayCircleFilled style={{ color: '#1890ff' }} />
      </Tooltip>
    ),
  },
  1: {
    value: '1',
    statusName: '进行中',
    color: 'green',
    icon: (
      <Tooltip title="点击完成">
        <IconFont type="icon-jinhangzhong-shijian-daojishi-03" style={{ color: 'green' }} />
      </Tooltip>
    ),
  },
  2: {
    value: '2',
    statusName: '已完成',
    color: 'grey',
    icon: (
      <Tooltip title="已完成">
        <IconFont type="icon-yiwancheng" style={{ color: 'grey' }} />
      </Tooltip>
    ),
  },
};

function getRandomColor() {
  let r = Math.floor(Math.random() * (255 - 150 + 1) + 150);
  let g = Math.floor(Math.random() * (255 - 150 + 1) + 150);
  let b = Math.floor(Math.random() * (255 - 150 + 1) + 150);
  return `rgb(${r}, ${g}, ${b})`;
}

function ProCard({ project_name, id, run, user_info_list, created_at, deadline, stastic, total }) {
  return (
    <Col {...col} style={{ marginBottom: 12 }}>
      <Card
        title={
          <div className="centerl">
            <span>{project_name}</span>
            <span>
              {dayjs(deadline).endOf('day').diff(dayjs(), 'hour') > 0 ? (
                dayjs(deadline).endOf('day').diff(dayjs(), 'hour') > 24 ? (
                  <b style={{ fontSize: 12, paddingLeft: 8, color: 'rgba(115,155,6,1)' }}>
                    距截止{dayjs(deadline).endOf('day').diff(dayjs(), 'day')}天
                  </b>
                ) : (
                  <b style={{ fontSize: 12, paddingLeft: 8, color: 'rgb(178, 116, 0)' }}>
                    距截止{dayjs(deadline).endOf('day').diff(dayjs(), 'hour')}小时
                  </b>
                )
              ) : (
                <b style={{ fontSize: 12, paddingLeft: 8, color: '#d84c00' }}>
                  超过截止{dayjs().diff(dayjs(deadline).endOf('day'), 'hour')}小时
                </b>
              )}
            </span>
          </div>
        }
        style={{ backgroundColor: 'rgba(255,255,255,0.25)', height: '100%' }}
        hoverable
        className="hovercard"
        extra={
          <Popconfirm
            title="是否删除该小组？"
            placement="bottomRight"
            onConfirm={(e) => {
              e.stopPropagation();
              doFetch({ url: '/webtool/v1/project/' + id, params: {}, method: 'DELETE' }).then(
                (res) => {
                  if (res.code === 0) {
                    message.success('删除成功');
                    run?.();
                  }
                },
              );
            }}
            onCancel={(e) => {
              e.stopPropagation();
            }}
          >
            <DeleteOutlined
              style={{ color: '#ff4800' }}
              onClick={(e) => {
                e.stopPropagation();
              }}
            />
          </Popconfirm>
        }
        onClick={() => {
          history.push('/welcome/project', {
            id,
          });
        }}
      >
        <div style={{ marginTop: -18, marginBottom: -8 }}>
          <div className="center">
            <span>小组进度: </span>
            <Tooltip placement="bottomRight" title="已完成任务数/全部任务数">
              <Progress
                percent={total ? ((stastic[stastic.length - 1].num.length ?? 0) / total) * 100 : 0}
                size="small"
                status="active"
                style={{ margin: 0, flex: 1, marginLeft: 8 }}
                strokeColor="rgba(64, 144, 255, 0.8)"
              />
            </Tooltip>
          </div>
          <div
            className="center whitebg"
            style={{
              justifyContent: 'space-between',
              color: '#666666',
              marginTop: 6,
              fontSize: 12,
              gap: 8,
            }}
          >
            {stastic?.map((item, index) => {
              return (
                <span style={{ color: items[item.status].color }}>
                  <b>{item.statusName}</b>
                  <b>{item?.num?.length ?? 0}</b>
                </span>
              );
            })}
          </div>
          <div className="spread" style={{ margin: '12px 0 -8px 0' }}>
            <Avatar.Group maxCount={6}>
              {user_info_list?.map((it, i) => {
                if (it?.head_url && it?.head_url !== '') {
                  return (
                    <Tooltip key={it?.id} title={it?.user_name}>
                      <Avatar src={it?.head_url} />
                    </Tooltip>
                  );
                } else {
                  return (
                    <Tooltip key={it?.id} title={it?.user_name}>
                      <Avatar>{it?.user_name?.charAt(0)}</Avatar>
                    </Tooltip>
                  );
                }
              })}
            </Avatar.Group>
            <Tooltip title={dayjs(created_at).format('YYYY-MM-DD HH:mm')}>
              <div className="center" style={{ color: '#999999' }}>
                <ClockCircleFilled style={{ marginRight: 4 }} />
                {dayjs().diff(dayjs(created_at), 'hour') > 24
                  ? `${dayjs().diff(dayjs(created_at), 'day')}天前`
                  : dayjs().diff(dayjs(created_at), 'hour') < 1
                  ? `${dayjs().diff(dayjs(created_at), 'minute')}分钟前`
                  : `${dayjs().diff(dayjs(created_at), 'hour')}小时前`}
              </div>
            </Tooltip>
          </div>
        </div>
      </Card>
    </Col>
  );
}
function AddCard({ onClick }) {
  return (
    <Col {...col} style={{ marginBottom: 12 }}>
      <Card
        onClick={onClick}
        style={{
          backgroundColor: 'rgba(255,255,255,0.25)',
          color: 'rgb(24, 144, 255)',
          height: '100%',
        }}
        hoverable
        className="center"
      >
        <PlusOutlined />
        <span style={{ paddingLeft: 12 }}>新建小组</span>
      </Card>
    </Col>
  );
}

function Homepage(props) {
  const {
    initialState: { currentUser, fetchUserInfo },
    setInitialState,
  } = useModel('@@initialState');
  const [modal, setmodal] = useState({
    open: false,
  });
  const { data, loading, run } = useRequest(
    async () => {
      let res = await getFetch({ url: '/webtool/v1/project' });
      return res?.data;
    },
    {
      debounceWait: 300,
      refreshDeps: [currentUser?.org_id],
    },
  );
  const [sort, setsort] = useState({
    time: false,
  });

  return (
    <div className="homepage">
      <div
        className="center bglight"
        style={{
          justifyContent: 'space-between',
          margin: '0px 0 12px 0',
          padding: 12,
          borderRadius: 12,
        }}
      >
        <div
          className="sort"
          onClick={() => {
            setsort((s) => ({
              ...s,
              time: !s.time,
            }));
          }}
        >
          <Tooltip title={sort.time ? '时间升序' : '时间降序'}>
            <ArrowUpOutlined rotate={sort.time ? 0 : 180} />
          </Tooltip>
        </div>
        <div className="hoverables" onClick={() => {}}>
          <Input
            prefix={<SearchOutlined />}
            placeholder="按照项目名称筛选"
            onChange={(e) => {
              setsort((s) => ({
                ...s,
                search: e.target.value,
              }));
            }}
          />
        </div>
      </div>

      <DragModal
        {...modal}
        onCancel={() => {
          setmodal((s) => ({
            ...s,
            open: false,
          }));
        }}
      >
        {modal?.title === '新建小组' && (
          <AddPro
            defaultid={currentUser?.id}
            refresh={() => {
              run();
              setmodal((s) => ({
                ...s,
                open: false,
              }));
            }}
          />
        )}
        {modal?.title === '加入组织' && (
          <Join
            refresh={() => {
              run();
              setmodal((s) => ({
                ...s,
                open: false,
              }));
            }}
          />
        )}
      </DragModal>
      <Skeleton active loading={loading}>
        <Row gutter={12}>
          <AddCard
            onClick={() => {
              setmodal((s) => ({
                ...s,
                open: true,
                title: '新建小组',
              }));
            }}
          />
          {data
            ?.sort((a, b) => {
              if (sort?.time) {
                return dayjs(a.created_at).valueOf() - dayjs(b.created_at).valueOf();
              } else {
                return dayjs(b.created_at).valueOf() - dayjs(a.created_at).valueOf();
              }
            })
            ?.filter((it) => {
              if (!sort?.search) {
                return true;
              }
              return it?.project_name?.indexOf(sort?.search) !== -1;
            })
            ?.map?.((it, i) => {
              return <ProCard run={run} {...it} key={i} />;
            })}
        </Row>
      </Skeleton>
    </div>
  );
}

export default Homepage;
