/* eslint-disable react/jsx-no-duplicate-props */
import DragModal from '@/components/DragModal';
import IconFont from '@/components/IconFont';
import { doFetch } from '@/utils/doFetch';
import { CloseOutlined, PlayCircleFilled, EllipsisOutlined } from '@ant-design/icons';
import { useModel } from '@umijs/max';
import { Avatar, Button, Empty, Modal, Popconfirm, Select, Tag, Tooltip } from 'antd';
import dayjs from 'dayjs';
import { memo, useState } from 'react';
import { Draggable, Droppable } from 'react-beautiful-dnd';
import styles from './index.less';

const items = {
  0: {
    value: '0',
    statusName: '未开始',
    icon: (
      <Tooltip title="点击开始">
        <PlayCircleFilled style={{ color: '#1890ff' }} />
      </Tooltip>
    ),
  },
  1: {
    value: '1',
    statusName: '进行中',
    color: 'green',
    icon: (
      <Tooltip title="点击完成">
        <IconFont type="icon-jinhangzhong-shijian-daojishi-03" style={{ color: 'green' }} />
      </Tooltip>
    ),
  },
  2: {
    value: '2',
    statusName: '已完成',
    color: 'grey',
    icon: (
      <Tooltip title="已完成">
        <IconFont type="icon-yiwancheng" style={{ color: 'grey' }} />
      </Tooltip>
    ),
  },
};

function getUserInfo(userList, id) {
  return userList?.filter((it) => it.id === id)[0];
}

const QuoteList = ({ listId, datas, add, edit, userList, refresh }) => {
  const [modal, setmodal] = useState({
    open: false,
  });
  const {
    initialState: { currentUser, fetchUserInfo },
    setInitialState,
  } = useModel('@@initialState');

  const Header = ({ userList, id, noname, click, size }) => {
    return (
      <div style={{ display: 'flex', alignItems: 'center', gap: 8 }}>
        <Tooltip title={getUserInfo(userList, id)?.user_name}>
          {getUserInfo(userList, id)?.head_url && getUserInfo(userList, id)?.head_url !== '' ? (
            <Avatar
              onClick={(e) => {
                e.stopPropagation();
                noname && click();
              }}
              size={size ?? 'large'}
              src={getUserInfo(userList, id)?.head_url}
            ></Avatar>
          ) : (
            <Avatar
              onClick={(e) => {
                e.stopPropagation();
                noname && click();
              }}
              size={size ?? 'large'}
            >
              {getUserInfo(userList, id)?.user_name?.charAt(0)}
            </Avatar>
          )}
        </Tooltip>
        {!noname && <b>{getUserInfo(userList, id)?.user_name}</b>}
      </div>
    );
  };

  return (
    <div>
      <DragModal
        {...modal}
        onCancel={() => {
          setmodal((s) => ({
            ...s,
            open: false,
          }));
        }}
        placement="left"
        width="100%"
        style={{ maxWidth: 580 }}
        extra={
          <CloseOutlined
            onClick={() => {
              setmodal((s) => ({
                ...s,
                open: false,
              }));
            }}
          />
        }
      >
        <div style={{ marginTop: 0 }}>
          <div className="spread">
            <Header size="small" userList={userList} id={modal?.userid}></Header>
            <div className="spread">
              {modal?.tags?.map((it) => {
                const isLongTag = it.tag_name.length > 20;
                return (
                  <Tag
                    key={it?.id}
                    style={{
                      userSelect: 'none',
                      display: 'flex',
                      alignItems: 'center',
                    }}
                    color={it?.color}
                  >
                    <div></div>
                    <span>{isLongTag ? `${it.tag_name.slice(0, 20)}...` : it.tag_name}</span>
                  </Tag>
                );
              })}
            </div>
          </div>

          {modal?.content ? (
            <div dangerouslySetInnerHTML={{ __html: modal?.content }}></div>
          ) : (
            <Empty description={false} style={{ marginTop: 12, marginBottom: 24 }}></Empty>
          )}

          <div className="center" style={{ justifyContent: 'space-between' }}>
            <span>截止日期：{dayjs(modal?.deadline).format('YYYY-MM-DD') ?? '未设置'}</span>
            <a
              style={{ textAlign: 'right', display: 'inline-block' }}
              onClick={async () => {
                await setmodal((s) => ({
                  ...s,
                  open: false,
                }));
                edit?.({
                  mission_name: modal?.title,
                  remark: modal?.content,
                  deadline: modal?.deadline,
                  userid: modal?.userid,
                  id: modal?.id,
                  tags: modal?.tags?.map((it) => it?.id),
                });
              }}
            >
              编辑
            </a>
          </div>
        </div>
      </DragModal>
      <Droppable droppableId={listId}>
        {(dropProvided, dropSnapshot) => (
          <div
            className={styles?.QuoteList}
            {...dropProvided.droppableProps}
            ref={dropProvided.innerRef}
          >
            {datas
              ?.sort((a, b) => a.sort - b.sort)
              .map((item, index) => (
                <Draggable key={item.id} draggableId={`item-${item.id}`} index={index}>
                  {(provided) => (
                    <div
                      className={styles?.item}
                      ref={provided.innerRef}
                      {...provided.draggableProps}
                      {...provided.dragHandleProps}
                      onClick={() => {
                        setmodal((s) => ({
                          ...s,
                          open: true,
                          title: item?.mission_name,
                          content: item?.remark,
                          deadline: item?.deadline,
                          userid: item?.userid,
                          id: item?.id,
                          tags: item?.tags,
                        }));
                      }}
                    >
                      <div className="spread">
                        <b style={{ fontSize: 15 }}>{item.mission_name}</b>
                        {currentUser?.id === item.userid && (
                          <Popconfirm
                            title="是否删除该任务？"
                            placement="bottomRight"
                            onConfirm={(e) => {
                              e.stopPropagation();
                              doFetch({
                                url: `/webtool/v1/item/${item.id}`,
                                params: {},
                                method: 'DELETE',
                              }).then((res) => {
                                refresh();
                              });
                            }}
                            onCancel={(e) => {
                              e.stopPropagation();
                            }}
                          >
                            <CloseOutlined
                              onClick={(e) => {
                                e.stopPropagation();
                              }}
                              style={{ fontSize: 12 }}
                            />
                          </Popconfirm>
                        )}
                      </div>
                      <div className="spread" style={{ gap: 12 }}>
                        <Header
                          userList={userList}
                          id={item?.userid}
                          noname={true}
                          click={() => {
                            Modal.confirm({
                              title: '修改执行人',
                              maskClosable: true,
                              content: (
                                <div>
                                  <Select
                                    style={{ width: '100%' }}
                                    placeholder={'修改执行人'}
                                    options={userList?.map((it) => {
                                      return {
                                        label: it?.user_name,
                                        value: it?.id,
                                      };
                                    })}
                                    onChange={(value) => {
                                      doFetch({
                                        url: '/webtool/v1/item/' + item.id,
                                        params: { userid: value },
                                        method: 'PUT',
                                      }).then((res) => {
                                        if (res.code === 0) {
                                          Modal.destroyAll();
                                          refresh();
                                        }
                                      });
                                    }}
                                  ></Select>
                                </div>
                              ),
                              footer: false,
                            });
                          }}
                        ></Header>
                        <div style={{ flex: 1 }}>
                          <div className="center" style={{ justifyContent: 'flex-start' }}>
                            {item?.tags
                              ?.filter((it, i) => i < 3)
                              .map((it) => {
                                const isLongTag = it.tag_name.length > 6;
                                return (
                                  <Tag
                                    key={it?.id}
                                    style={{
                                      userSelect: 'none',
                                      display: 'flex',
                                      alignItems: 'center',
                                    }}
                                    color={it?.color}
                                  >
                                    <div></div>
                                    <span>
                                      {isLongTag ? `${it.tag_name.slice(0, 6)}...` : it.tag_name}
                                    </span>
                                  </Tag>
                                );
                              })}
                            {
                              item.tags.length>3 && <EllipsisOutlined/>
                            }

                          </div>
                          <div
                            className="spread"
                            style={{ marginTop: 6, alignItems: 'flex-start' }}
                            onClick={(e) => {
                              e.stopPropagation();
                            }}
                          >
                            <div>
                              <b style={{ color: items[item?.status]?.color, paddingRight: 8 }}>
                                {items[item?.status]?.statusName}
                              </b>
                              {item.deadline ? (
                                <Tooltip title={dayjs(item.deadline).format('YYYY-MM-DD')}>
                                  <span style={{ fontSize: 12 }}>
                                    {dayjs(item.deadline).endOf('day').diff(dayjs(), 'day') > 0 ? (
                                      `剩余${dayjs(item.deadline)
                                        .endOf('day')
                                        .diff(dayjs(), 'day')}天`
                                    ) : dayjs(item.deadline).endOf('day').diff(dayjs(), 'day') ===
                                      0 ? (
                                      '今天截止'
                                    ) : (
                                      <span style={{ color: 'red' }}>
                                        超时{dayjs().diff(dayjs(item.deadline).endOf('day'), 'day')}
                                        天
                                      </span>
                                    )}
                                  </span>
                                </Tooltip>
                              ) : (
                                <span style={{ fontSize: 12 }}>未设置截止时间</span>
                              )}
                            </div>
                            <Button
                              size="small"
                              onClick={async () => {
                                let res = {};
                                if (item.status === '0') {
                                  res = await doFetch({
                                    url: '/webtool/v1/item/' + item.id,
                                    params: { status: '1' },
                                    method: 'PUT',
                                  });
                                } else if (item.status === '1') {
                                  res = await doFetch({
                                    url: '/webtool/v1/item/' + item.id,
                                    params: { status: '2' },
                                    method: 'PUT',
                                  });
                                }
                                if (res.code === 0) {
                                  refresh();
                                }
                              }}
                              icon={items[item?.status]?.icon}
                            ></Button>
                          </div>
                        </div>
                      </div>
                    </div>
                  )}
                </Draggable>
              ))}
            <div
              key="add"
              className={styles?.item}
              onClick={() => {
                add?.();
              }}
              style={{
                display: 'flex',
                justifyContent: 'center',
                color: '#000000',
                userSelect: 'none',
              }}
            >
              新建任务
            </div>
          </div>
        )}
      </Droppable>
    </div>
  );
};

export default memo(QuoteList, (prev, next) => {
  if (JSON.stringify(prev.datas) === JSON.stringify(next.datas)) {
    return true;
  }
  return false;
});
